-- =============================================================================================================
-- *
-- * Copyright (c) University of York
-- *
-- * File Name: computer_TB.vhd
-- *
-- * Version: V1.0
-- *
-- * Release Date: 
-- *
-- * Author(s): M.Freeman
-- *
-- * Description: testbench
-- *
-- * Change History:  $Author: $
-- *                  $Date: $
-- *                  $Revision: $
-- *
-- * Conditions of Use: THIS CODE IS COPYRIGHT AND IS SUPPLIED "AS IS" WITHOUT WARRANTY OF ANY KIND, INCLUDING,
-- *                    BUT NOT LIMITED TO, ANY IMPLIED WARRANTY OF MERCHANTABILITY AND FITNESS FOR A
-- *                    PARTICULAR PURPOSE.
-- *
-- * Notes:
-- *   
-- =============================================================================================================

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

LIBRARY UNISIM;
USE UNISIM.Vcomponents.ALL;

ENTITY computer_tb IS
END computer_tb;

ARCHITECTURE computer_tb_arch OF computer_tb IS 

   COMPONENT computer_debug
   PORT( 
	  CLK	: IN  STD_LOGIC; 
     CLR	: IN  STD_LOGIC; 
     RX  : IN  STD_LOGIC;			
     TX  : OUT STD_LOGIC;
     TP0 : OUT STD_LOGIC;			
     TP1 : OUT STD_LOGIC;
     TP2 : OUT STD_LOGIC );
   END COMPONENT;

   COMPONENT debug
   PORT( 
	  CLK	 : IN  STD_LOGIC; 
     CLR	 : IN  STD_LOGIC; 
     ADDR : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);			
     DATA : IN  STD_LOGIC_VECTOR(15 DOWNTO 0) );
   END COMPONENT;
	
   COMPONENT serial_rx_port 
   GENERIC (
     delay : TIME := 100 ns;
     save_file : STRING := "data.txt" );
   PORT (
     sin  : IN STD_LOGIC;
     dout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
     char : OUT String(1 to 1);
     tp   : OUT STD_LOGIC );
   END COMPONENT;

   SIGNAL CLK	  : STD_LOGIC;
   SIGNAL CLR	  : STD_LOGIC;
	
   SIGNAL RX	  : STD_LOGIC;
   SIGNAL TX     : STD_LOGIC;
   SIGNAL TP0	  : STD_LOGIC;
   SIGNAL TP1    : STD_LOGIC;
   SIGNAL TP2    : STD_LOGIC;
	
   SIGNAL value  : STD_LOGIC_VECTOR(7 DOWNTO 0);
   SIGNAL letter : String(1 to 1);
   SIGNAL sample : STD_LOGIC;

BEGIN

   UUT: computer_debug PORT MAP(
		CLK => CLK, 
		CLR => CLR, 
      RX  => RX, 		
      TX  => TX, 
      TP0 => TP0, 			
      TP1 => TP1,
		TP2 => TP2
   );
	
   UART_RX : serial_rx_port
   GENERIC MAP(
     delay => 3.333333333 ms,
     save_file => "dataOut.txt" )
   PORT MAP(
     sin  => TX,
     dout => value,
     char => letter,
     tp   => sample );
	  
   clock : PROCESS
   BEGIN
      CLK <= '0'; wait for 50 ns;
      CLK <= '1'; wait for 50 ns;		
   END PROCESS;
	
   clear : PROCESS
   BEGIN
      CLR <= '1'; wait for 150 ns;
      CLR <= '0'; wait;		
   END PROCESS;
  
   tb : PROCESS
   BEGIN
     RX <= '1'; 
	  wait;
   END PROCESS; 
	 
END computer_tb_arch;
